unit SafeWS;

{ Safe TWinSocketStream.

  Author: Primoz Gabrijelcic

  Code accompanying an article in The Delphi Magazine. May be used as-is in your
  own application. 
}

interface

uses
  ScktComp;

type
  TSafeWinSocketStream = class(TWinSocketStream)
    constructor Create(ASocket: TCustomWinSocket; TimeOut: longint);
    destructor  Destroy; override;
    function    Read(var Buffer; Count: longint): longint; override;
    function    Write(const Buffer; Count: longint): longint; override;
  private
    swsFailed: boolean;
    swsBuffer: pointer;
  end;

implementation

const
  CSmallBlockSize = 1024;

  constructor TSafeWinSocketStream.Create(ASocket: TCustomWinSocket; TimeOut: longint);
  begin
    swsFailed := false;
    GetMem(swsBuffer,CSmallBlockSize);
    inherited Create(ASocket, TimeOut);
  end; { TSafeWinSocketStream.Create }

  destructor TSafeWinSocketStream.Destroy;
  begin
    inherited Destroy;
    FreeMem(swsBuffer);
  end; { TSafeWinSocketStream.Destroy }

  function TSafeWinSocketStream.Read(var Buffer; Count: longint): longint;
  var
    numb,lread: integer;
  begin
    if swsFailed then Result := 0
    else begin
      try
        numb := 0;
        lread := 1;
        while (lread > 0) and (numb < count) do begin
          lread := count-numb;
          if lread > CSmallBlockSize then lread := CSmallBlockSize; {use small blocks}
          if not inherited WaitForData(TimeOut) then lread := 0
          else lread := inherited Read(swsBuffer^,lread);
          if lread > 0 then begin
            Move(swsBuffer^,pointer(integer(@buffer)+numb)^,lread);
            numb := numb + lread;
          end;
        end;
        Result := numb;
      except Result := 0; end;
      swsFailed := (Result <> count);
    end;
  end; { TSafeWinSocketStream.Read }

  function TSafeWinSocketStream.Write(const Buffer; Count: longint): longint;
  var
    numb,lwrite: integer;
  begin
    if swsFailed then Result := 0
    else begin
      try
        numb := 0;
        lwrite := 1;
        while (lwrite > 0) and (numb < count) do begin
          lwrite := count-numb;
          if lwrite > CSmallBlockSize then lwrite := CSmallBlockSize; {use small blocks}
          Move(pointer(integer(@buffer)+numb)^,swsBuffer^,lwrite);
          lwrite := inherited Write(swsBuffer^,lwrite);
          numb := numb + lwrite;
        end;
        Result := numb;
      except Result := 0; end;
      swsFailed := (Result <> count);
    end;
  end; { TSafeWinSocketStream.Write }

end.
